@@ -76,13 +76,12 @@ typedef struct {
 	struct list_head list;
 } bmm_block_t;
 
-static unsigned long bmm_vaddr;
 static unsigned long bmm_paddr;
 static unsigned long bmm_size;
 static unsigned long bmm_free_size;
 static bmm_block_t bmm_free_block;
 static bmm_block_t bmm_used_block;
-static spinlock_t bmm_lock;	/* spin lock for block list */
+static struct mutex bmm_mutex;	/* mutex for block list */
 
 #define UNUSED_PARAM(x)	(void)(x)
 
@@ -132,7 +131,7 @@ unsigned long va_to_pa(unsigned long user_addr, unsigned int size)
 	int i = 0;
 	struct mm_struct *mm = current->mm;
 
-	if(vaddr == NULL)
+	if(vaddr == 0)
 		return 0;
 
 	paddr = uva_to_pa(mm, vaddr);
@@ -177,68 +176,64 @@ static void bmm_dump_all(void)
 
 static bmm_block_t * bmm_search_vaddr(struct list_head *head, unsigned long vaddr)
 {
-	unsigned long flags;
 	bmm_block_t *pbmm;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, head, list) {
 		if(pbmm->vaddr == vaddr && pbmm->pid == current->tgid) {
-			spin_unlock_irqrestore(&bmm_lock, flags);
+			mutex_unlock(&bmm_mutex);
 			return pbmm;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	return NULL;
 }
 
 static bmm_block_t * bmm_search_vaddr_ex(struct list_head *head, unsigned long vaddr)
 {
-	unsigned long flags;
 	bmm_block_t *pbmm;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, head, list) {
 		if(pbmm->vaddr <= vaddr && pbmm->vaddr + pbmm->size > vaddr && pbmm->pid == current->tgid) {
-			spin_unlock_irqrestore(&bmm_lock, flags);
+			mutex_unlock(&bmm_mutex);
 			return pbmm;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	return NULL;
 }
 
 static bmm_block_t * bmm_search_paddr_ex(struct list_head *head, unsigned long paddr)
 {
-	unsigned long flags;
 	bmm_block_t *pbmm;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, head, list) {
 		if(pbmm->paddr <= paddr && pbmm->paddr + pbmm->size > paddr) {
-			spin_unlock_irqrestore(&bmm_lock, flags);
+			mutex_unlock(&bmm_mutex);
 			return pbmm;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	return NULL;
 }
 
 static bmm_block_t * bmm_search_paddr(struct list_head *head, unsigned long paddr)
 {
-	unsigned long flags;
 	bmm_block_t *pbmm;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, head, list) {
 		if(pbmm->paddr == paddr) {
-			spin_unlock_irqrestore(&bmm_lock, flags);
+			mutex_unlock(&bmm_mutex);
 			return pbmm;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	return NULL;
 }
@@ -306,13 +301,12 @@ static unsigned long bmm_malloc(size_t size, unsigned long attr)
 	bmm_block_t *pbmm;
 	bmm_block_t *new;
 	unsigned long paddr = 0;
-	unsigned long flags;
 
 	size = PAGE_ALIGN(size);
 	if(size == 0 || size > bmm_free_size || size > TASK_SIZE)
 		return 0;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, &(bmm_free_block.list), list) {
 		if(pbmm->size == size) {	/* Found an exact match */
 			list_del_init(&(pbmm->list));
@@ -340,7 +334,7 @@ static unsigned long bmm_malloc(size_t size, unsigned long attr)
 			break;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	pr_debug("bmm_malloc return paddr=0x%08lx\n", paddr);
 
@@ -349,13 +343,12 @@ static unsigned long bmm_malloc(size_t size, unsigned long attr)
 
 static void bmm_free(unsigned long vaddr)
 {
-	unsigned long flags;
 	bmm_block_t *pbmm;
 	pid_t pid = current->tgid;
 
 	pr_debug("bmm_free(vaddr=0x%08lx) pid=%d\n", vaddr, pid);
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, &(bmm_used_block.list), list) {
 		pr_debug("\t(vaddr=0x%08lx, pid=0x%08x)\n", pbmm->vaddr, pbmm->pid);
 		if(pbmm->vaddr == vaddr && pbmm->pid == pid) {
@@ -365,14 +358,42 @@ static void bmm_free(unsigned long vaddr)
 			pbmm->vaddr = 0;
 			pbmm->attr = 0;
 			pbmm->pid = 0;
-			bmm_insert(&(bmm_free_block.list), pbmm, 1);
 			bmm_free_size += pbmm->size;
+			bmm_insert(&(bmm_free_block.list), pbmm, 1);
 			break;
 		}
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 }
 
+unsigned long bmm_malloc_kernel(size_t size, unsigned long attr)
+{
+	return bmm_malloc(size, attr);
+}
+EXPORT_SYMBOL(bmm_malloc_kernel);
+
+void bmm_free_kernel(unsigned long paddr)
+{
+	bmm_block_t *pbmm;
+
+	mutex_lock(&bmm_mutex);
+	list_for_each_entry(pbmm, &(bmm_used_block.list), list) {
+		if(pbmm->paddr == paddr) {
+			pr_debug("\tbmm_free(paddr=0x%08lx)\n", pbmm->paddr);
+
+			list_del_init(&(pbmm->list));
+			pbmm->vaddr = 0;
+			pbmm->attr = 0;
+			pbmm->pid = 0;
+			bmm_free_size += pbmm->size;
+			bmm_insert(&(bmm_free_block.list), pbmm, 1);
+			break;
+		}
+	}
+	mutex_unlock(&bmm_mutex);
+}
+EXPORT_SYMBOL(bmm_free_kernel);
+
 static void bmm_vm_close(struct vm_area_struct *vma)
 {
 	pr_debug("vm_close(%p)\n", vma);
@@ -426,16 +447,15 @@ static int bmm_mmap(struct file *file, struct vm_area_struct *vma)
 
 static unsigned long bmm_get_allocated_size(void)
 {
-	unsigned long flags;
 	unsigned long size = 0;
 	bmm_block_t *pbmm;
 
-	spin_lock_irqsave(&bmm_lock, flags);
+	mutex_lock(&bmm_mutex);
 	list_for_each_entry(pbmm, &(bmm_used_block.list), list) {
 		if(pbmm->pid == current->tgid)
 			size += pbmm->size;
 	}
-	spin_unlock_irqrestore(&bmm_lock, flags);
+	mutex_unlock(&bmm_mutex);
 
 	return size;
 }
@@ -555,10 +575,10 @@ void bmm_consistent_sync(unsigned long start, size_t size, int direction)
 	paddr = bmm_get_paddr_inside_ex(start);
 
 	/* kernel space */
-	if (paddr == NULL)
+	if (paddr == 0)
 		paddr = bmm_get_paddr_ex(start, size);
 
-	if (paddr == NULL)
+	if (paddr == 0)
 		BUG();
 
 	switch (direction) {
@@ -878,27 +898,25 @@ static struct miscdevice bmm_misc = {
 static int __bmm_init(void)
 {
 	struct page *page;
-	unsigned long addr, size;
+	unsigned long size;
 
 	bmm_size = bmm_size_mb * 1024 * 1024;
 	pr_debug("Trying to allocate %ldMB memory with an order=%d\n", bmm_size_mb, get_order(bmm_size));
 	
-	page = alloc_pages(GFP_KERNEL, get_order(bmm_size));
+	page = alloc_pages(GFP_KERNEL | __GFP_HIGHMEM, get_order(bmm_size));
 	if (page == NULL){
 		printk(KERN_ERR "Error allocating memory %ldMB\n", bmm_size_mb);
 		return -ENOMEM;
 	}
 	
-	bmm_vaddr = (unsigned long)page_address(page);
-	bmm_paddr = virt_to_phys((void *)bmm_vaddr);
+	bmm_paddr = page_to_pfn(page) << PAGE_SHIFT;
 
-	pr_debug("page = 0x%08lx, va = 0x%08lx, pa = 0x%08lx\n", (unsigned long)page, bmm_vaddr, bmm_paddr);
+	pr_debug("bmm_init: allocate page = 0x%08lx, pa = 0x%08lx\n", (unsigned long)page, bmm_paddr);
 
-	addr = bmm_vaddr;
 	size = bmm_size;
 	while(size > 0) {
-		SetPageReserved(virt_to_page(addr));
-		addr += PAGE_SIZE;
+		SetPageReserved(page);
+		page++;
 		size -= PAGE_SIZE;
 	}
 	return 0;
@@ -906,19 +924,24 @@ static int __bmm_init(void)
 
 static void __bmm_exit(void)
 {
-	unsigned long addr, size;
+	unsigned long size;
+	struct page *page;
 
 	pr_debug("BMM free memory\n");
 
-	addr = bmm_vaddr;
+	page = pfn_to_page(bmm_paddr >> PAGE_SHIFT);
 	size = bmm_size;
 	while(size > 0) {
-		ClearPageReserved(virt_to_page(addr));
-		addr += PAGE_SIZE;
+		ClearPageReserved(page);
+		page++;
 		size -= PAGE_SIZE;
 	}
 
-	__free_pages(virt_to_page(bmm_vaddr), get_order(bmm_size));
+	page = pfn_to_page(bmm_paddr >> PAGE_SHIFT);
+
+	pr_debug("bmm_exit: free page = 0x%08lx, pa = 0x%08lx, size=0x%08lx\n", (unsigned long)page, bmm_paddr, bmm_size);
+
+	__free_pages(page, get_order(bmm_size));
 }
 #else
 static int __bmm_init(void)
@@ -962,7 +985,7 @@ static int __init bmm_init(void)
 	if(IS_ERR((void *)__bmm_init()))
 		return -ENOMEM;
 
-	spin_lock_init(&bmm_lock);
+	mutex_init(&bmm_mutex);
 
 	bmm_free_size = bmm_size;
 	INIT_LIST_HEAD(&(bmm_free_block.list));
